/*
 *
 * URI.h
 *
 * Copyright (c) 2003 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef EDGReplicaManager_URI_H
#define EDGReplicaManager_URI_H 1

/*
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @version $Id: URI.h,v 1.4 2004/06/14 12:01:17 jamesc Exp $
 */

#include <string>
#include <deque>

#include "EdgReplicaManager/ReplicaManagerExceptions.h"

namespace EdgReplicaManager {

    //! A simple URI class, incomplete but fits our needs
    class URI {
    public:

	/** Constructor from string
	 */

	URI(const std::string &uri) throw(URIException);

	/** Constructor from scheme, scheme-specific-part
	 */

	URI(const std::string &scheme,
	    const std::string &ssp) throw(URIException);

	/** Constructor using scheme, host, path
	 */

	URI(const std::string &scheme,
	    const std::string &host,
	    const std::string &path) throw(URIException);

	/** Constructor using scheme, host, port, path
	 */

	URI(const std::string &scheme,
	    const std::string &host,
	    int port,
	    const std::string &path) throw(URIException);

	/** Standard virtual destructor
	 */
	virtual ~URI() {}

	/**
	 * return the host of the given URI
	 *
	 */
	std::string getHost() throw(URIException);

	/**
	 * return the scheme of the given URI
	 *
	 */
	std::string getScheme();

	/**
	 * return the scheme-specific part of the given URI
	 *
	 */
	std::string getSchemeSpecificPart();

	/**
	 * return the path of the given URI
	 *
	 */
	std::string getPath() throw(URIException);

	/**
	 * return the string form of the URI
	 *
	 */
	std::string toString();


	/**
	 * return the port of the given URI. returns -1 if not given.
	 *
	 */
	int getPort();

    protected:

	std::string m_scheme;
	std::string m_host;
	std::string m_path;
	std::string m_ssp;
	int m_port;
	std::string m_str;

	bool m_haveHost;
	bool m_havePath;

    private:

	// disallow default empty construction
	URI();

	// validators
	void validateScheme() throw(URIException);
	void validateSSP() throw(URIException);
	void validateHost() throw(URIException);
	void validatePath() throw(URIException);

	// helpers
	int getPort(std::string &port) throw(URIException);
	std::string hostPort();
	void checkForIllegalCharactersLazy(const std::string &str) throw(URIException);
	void checkForIllegalCharactersStrict(const std::string &str) throw(URIException);
	void normalizeHostPath(const std::string &path) throw(URIException);
	void doNormalize(std::deque<std::string> &dirs);
	std::string getPath(std::deque<std::string> &dirs);
	std::string normalizePath();
	void split(char c, const std::string &str, std::deque<std::string> &list);


    };

};

#endif
